////////////////////////////////////////////////////////////////////////////////
// TSplitter
//
// Controls that allows to resize two panels (or so) which are separated by a
// this control
//
// by Alan GARNY
//    gry@physiol.ox.ac.uk
//    http://www.physiol.ox.ac.uk/~gry
//
//  Copyright 1996-97
////////////////////////////////////////////////////////////////////////////////

#ifndef SplitterH
#define SplitterH

////////////////////////////////////////////////////////////////////////////////

enum TSplitterType {tyHorizontal, tyVertical};
enum TSplitterTypeAlign {taTopOrLeft, taBottomOrRight};

////////////////////////////////////////////////////////////////////////////////

extern const int _NO_MIN_MAX_SIZE_;

////////////////////////////////////////////////////////////////////////////////

class TSplitter: public TCustomPanel
{
   private:
      // Private representation of published properties

      TSplitterType FType;   // Type of the splitter (i.e. vertical or
                             // horizontal)
      TSplitterTypeAlign FTypeAlign;   // Specifies the type of alignment

      TControl *FControl1;   // Control which the "Align" property is set to
                             // "alLeft" or "alTop"
      TControl *FControl2;   // Control which the "Align" property is set to
                             // "alClient"

      int FMin, FMax;   // Minimum and maximum size "Control1" can have
                        // Note: a value of "-1" means that there is no
                        //       theoritical limit

      bool FRealTime;   // Allows to split in real time

      // Properties used for internal purpose

      int SplitterPos;   // Relative current position of the mouse when
                         // splitting (relative to OrigPos) 
      int OrigPos;   // Absolute original position of the mouse when starting
                     // the splitting
      HDC DC;   // Device context to draw the splitter
      int RealMin, RealMax;   // Very useful, in case there is no min/max.
      int GlobalWidth, GlobalHeight;   // Width/Height of "Control1", the
                                       // splitter and "Control2"

      // Methods to modify the different published properties

      void __fastcall SetType(const TSplitterType aValue);
      void __fastcall SetTypeAlign(const TSplitterTypeAlign aValue);
      void __fastcall SetControl1(TControl *aValue);
      void __fastcall SetControl2(TControl *aValue);
      TControl* __fastcall GetControl1();
      TControl* __fastcall GetControl2();
      void __fastcall SetMin(const int aValue);
      void __fastcall SetMax(const int aValue);
      void __fastcall SetRealTime(const bool aValue);

      // Methods used for internal purpose

      void __fastcall DrawSplitter();
      inline int __fastcall MaxSize();
      void __fastcall UpdateControls();
      bool __fastcall ValidControls();
      void __fastcall SimulateNullSplitting();

      // Overridden methods

      virtual void __fastcall Loaded();

   protected:
      // Methods to handle the mouse click on the control

      virtual void __fastcall MouseDown(TMouseButton aButton,
                                        TShiftState aShift, int aX, int aY);
      virtual void __fastcall MouseMove(TShiftState aShift, int aX, int aY);
      virtual void __fastcall MouseUp(TMouseButton aButton, TShiftState aShift,
                                      int aX, int aY);

      // Special internal events

      TNotifyEvent FOnStartSplit;
      TNotifyEvent FOnEndSplit;

   public:
      // Constructor & Destructor

      virtual __fastcall TSplitter(TComponent *aOwner);
      virtual __fastcall ~TSplitter();

   __published:
      // Published properties

      __property BevelInner;
      __property BevelOuter;
      __property BevelWidth;
      __property BorderStyle;
      __property BorderWidth;
      __property Color;
      __property TControl *Control1 = {read=GetControl1, write=SetControl1};
      __property TControl *Control2 = {read=GetControl2, write=SetControl2};
      __property Enabled;
      __property int Min = {read=FMin, write=SetMin};
      __property int Max = {read=FMax, write=SetMax};
      __property ParentColor;
      __property ParentShowHint;
      __property PopupMenu;
      __property bool RealTime = {read=FRealTime, write=SetRealTime, default=false};
      __property ShowHint;
      __property TSplitterType Type = {read=FType, write=SetType, default=tyVertical};
      __property TSplitterTypeAlign TypeAlign = {read=FTypeAlign, write=SetTypeAlign, default=taTopOrLeft};
      __property Visible;

      // Splitter events

      __property TNotifyEvent OnStartSplit = {read=FOnStartSplit, write=FOnStartSplit};
      __property TNotifyEvent OnEndSplit = {read=FOnEndSplit, write=FOnEndSplit};
};

////////////////////////////////////////////////////////////////////////////////

#endif

// End of file
////////////////////////////////////////////////////////////////////////////////
